#!/bin/bash
#
# script name:  chk_ip.sh
# purpose:      checking outbound link and change route and dns.
# author:       netman(netman@study-area.org)
# license:      GPL(http://www.gnu.org/licenses/gpl.html)
#
# date:         2004-09-23
# version:      v0.04
#
# caveate:
#       1) tested on redhat 9.0 linux only
#       2) using PING for link detection
#       3) iproute program is required
#       4) scripts run_ip.sh & ddns.sh are required too
#
#------------------------------------------------------------
# change log:
#       1) 2004-03-23 v0.01
#               * first beta release
#       2) 2004-03-24 v0.02
#               * add script path detection
#       4) 2004-07-07 v0.03
#               * IMPROVEMENT
#                 - Create central config file
#               * Add function:
#                 - Add email notification
#       5) 2004-09-23 v0.04
#               * IMPROVEMENT
#                 - Rewrite the method of getting dead interface
#
#------------------------------------------------------------

WD=${0%/*}
CONFIG=$WD/configs
[ -f $CONFIG ] && . $CONFIG || {
        echo "${0#*/}: ERROR: can not load $CONFIG."
        exit 1
}

# change working directory
cd $WD

#-- determine gateways --#
get_gw () {
        defaults=$($IP route show | $SED -n '/default/,$p' | $GREP via)
        if_n_gw=$(echo -en "$defaults" | $AWK '{print $5":"$3":'$fixed_dest'"}')
        if [ -n "$fixed_gw" ]; then
		fixed_pair=$(echo -e "${fixed_gw// /\n}" | awk -F: '{print $1":"$2":'$fixed_dest'"}')
		dyn_pair=$(echo "$if_n_gw" | $GREP -Ev "$(echo $fixed_if | $TR ' ' '|')")
        	all_pair=$(echo $fixed_pair $dyn_pair)
	else
        	all_pair=$(echo "$if_n_gw")
	fi
}

#-- configure static route --#
get_dev () {
        for i in "$@"; do
                if=${i%%:*}
                gw=$(echo $i | $CUT -d: -f2)
                dest=${i##*:}
                $IP route replace $dest via $gw dev $if && \
			sleep 1; $PING -c1 -w2 $dest &>/dev/null || echo $if
                $IP route del $dest via $gw dev $if
        done
        unset i
}

#-- run loop --#
while : ; do
        get_gw                  		# have gw
        d_dev=$(echo $(get_dev $all_pair))	# have dead link

        [ "$d_dev" ] && {			# send mail if dead link found
                echo "dead link found: $d_dev" | \
                mail -s "$(hostname): dead link" $MAIL
        }

        $SH $RUN_SCRIPT -d "$d_dev"		# to run ip script

        [ -f "$DDNS_SCRIPT"  -a "$DDNS" = yes ] && $SH $DDNS_SCRIPT

        unset all_pair
        sleep $INTERVAL
done

