#!/bin/bash
# 
# script name:	ddns.sh
# purpose:	updating dns record
# author:	netman(netman@study-area.org)
# license:	GPL(http://www.gnu.org/licenses/gpl.html)
#
# date:		2004-09-25
# version:	v0.04
#
# caveate:
#	1) tested on redhat 9.0 linux only.
#	2) iproute program is required.
#	3) a php webpage to show ip, and the lynx program are required.
#	4) ddns server and ddns key are required.
#
#------------------------------------------------------------
# change log:
#	1) 2004-03-23 v0.01
#		* first beta release
#	2) 2004-03-23 v0.02
#		* add EXCL_IF list for exception
#	3) 2004-03-23 v0.03
#		* add -e option for appending EXCL_IF
#	4) 2004-09-25 v0.04
#		* BugFix: detect gw interface list
#
#------------------------------------------------------------

# set commands
HOST=/usr/bin/host
GREP=/bin/grep
CAT=/bin/cat
IP=/sbin/ip
SED=/bin/sed
AWK=/bin/awk
LYNX=/usr/bin/lynx
TOUCH=/bin/touch
DIFF=/usr/bin/diff
NSUPDATE=/usr/bin/nsupdate
MV=/bin/mv

# set variables
if echo $0 | $GREP '^/' ; then
    w_dir=${0%/*}
else
    w_dir=$PWD/${0%/*}
fi
KEY_FILE=$w_dir/Ktestddns.+157+14615.key
UPDATE_DATA=$w_dir/nsupdate.data
UPDATE_OLD=$UPDATE_DATA.old
HOST_NAME=testddns.study-area.org
NS_SERVER=dns.study-area.org	# don't use IP address
IP_SERVER=dns.study-area.org	# server running ip.php program
IP_SERVER_IP=$(echo $($HOST $IP_SERVER) | tail -n 1 | $AWK '{print $NF}')
IP_URL="http://$IP_SERVER/ip.php"
CLIENT_TYPE="client"	# no proxy or using ISP's proxy
#CLIENT_TYPE="proxy"	# use local proxy
MASQ_IF="eth1"		# which masqueraded behine NAT
#EXCL_IF="eth1"		# note: also use -e options to append list

# have exception
while getopts "e:" opt; do
    case $opt in
	e) EXCL_IF=$(echo $EXCL_IF $OPTARG) ;;
    esac
done

# ensure key files
for file in $KEY_FILE ${KEY_FILE%key}private
do
    if [ ! -r $file ]; then
	echo "$(basename $0): ERROR: $file is not readable."
	exit 1
    fi
done

# ensure the server is connectable
$HOST $NS_SERVER $NS_SERVER | $GREP -q "^$NS_SERVER has address" || {
    echo "$(basename $0): ERROR: could not contact nameserver $NS_SERVER."
    exit 2
}

# prepare initial script
$CAT >| $UPDATE_DATA <<END
server $NS_SERVER
update delete $HOST_NAME A
END
test "$?" = "0" || {
    echo "$(basename $0): ERROR: could not create $UPDATE_DATA."
    exit 3
}

# get interfaces
ALL_IF=$($IP route show | $SED -n '/default/,$p' | $AWK '/via/{print $5}')
[ "$EXCL_IF" ] && ALL_IF=$(echo "$ALL_IF" | $GREP -Ev "${EXCL_IF// /|}")

# get masqueraded ip
get_mip () {
    for M_IF in $MASQ_IF; do
	echo $ALL_IF | $GREP -qw "$M_IF" || continue
	m_gw=$($IP route show | $SED -n '/default/,$p' |\
		$AWK '/'$M_IF'/{print $3}')
	$IP route replace $IP_SERVER_IP via $m_gw
	sleep 2
	M_IP=$($LYNX --dump $IP_URL | $GREP "$CLIENT_TYPE" |\
		$GREP -Eo '([0-9]{1,3}\.){3}[0-9]+')
	$IP route del $IP_SERVER_IP via $m_gw
	[ -z "$M_IP" ] && {
	    echo "${0##*/}: Error: can't determine IP for $M_IF."
	    exit 4
	}
	echo $M_IF:$M_IP
	unset m_gw M_IP
    done
}

# get current ip
for IF in $ALL_IF; do
    if echo "$MASQ_IF" | $GREP -wq "$IF"; then
	NEW_IP=$(get_mip | $AWK -F: '/'$IF'/{print $2}')
    else
	NEW_IP=$($IP address show dev $IF | $AWK '/inet/{print $2}')
    fi
    NEW_IP=$(echo $NEW_IP | $GREP -Eo '([0-9]{1,3}\.){3}[0-9]+')
    [ "$NEW_IP" ] && echo "update add $HOST_NAME 0 A $NEW_IP"
    unset NEW_IP
done >> $UPDATE_DATA
echo "send" >> $UPDATE_DATA

# do a test then update
$TOUCH $UPDATE_OLD
if ! $DIFF $UPDATE_OLD $UPDATE_DATA; then
	$NSUPDATE -k $KEY_FILE -v $UPDATE_DATA && {
		$MV $UPDATE_DATA $UPDATE_OLD
	}
fi

#-- end of script --#

